@@ -2,9 +2,32 @@ package Benno;
 use strict;
 use Digest::SHA qw(sha256);
 
+our $VERSION = '2.10.0';
 
 
-=head1 digest($filecontent,@secretheaders)
+=head2 verify_journaladdress($journaladdress)
+
+    Verifies the given journal email address and returns the GW header
+    part if successful.
+
+    $x_gwheader = Benno::verify_journaladdress($journaladdress);
+
+=cut
+sub verify_journaladdress
+{
+    my ($address) = @_;
+
+    my ($subscr,$ckext,$salt) = $address =~ /^(\S+?)\+(\S+?)(\S{4})\@/;
+
+    my $ckcmp = sprintf("%x",Benno::crc32($subscr.$salt));
+    if ($ckext == $ckcmp) {
+        return $subscr;  # X-BENNO-GW header value
+    }
+    return 0;
+}
+
+
+=head2 digest($filecontent,@secretheaders)
 
     Returns the digest of the file without secretheaders.
 
@@ -14,13 +37,12 @@ sub digest
     my ($class,$eml,@secretheader) = @_;
 
     my $ctx = Digest::SHA->new('256');
-
     my $in_header = 1;
-    foreach my $line (split /\R/, $eml) {
+    foreach my $line (split /^/m, $eml) {           # do not remove line endings
         if ($line =~ /^\R/) { $in_header = 0; }
         if ($in_header) {
-            my ($headername) = $line =~ /^(\S+):/;
-            if (grep /$headername/, @secretheader) {
+            my ($headername) = $line =~ /^(\S+?):/;
+            if (grep /^$headername$/i, @secretheader) {
                 next;
             }
         }
@@ -31,7 +53,7 @@ sub digest
 }
 
 
-=head1 id2box($bennoid)
+=head2 id2box($bennoid)
 
     Returns the box id of the given bennoid.
 
@@ -47,7 +69,7 @@ sub id2box
 }
 
 
-=head1 id2path($bennoid[$pathconfig])
+=head2 id2path($bennoid[$pathconfig])
 
     Returns the path representation of the given bennoid.
 
@@ -76,7 +98,7 @@ sub id2path
 
 
 
-=head1 config($configfile)
+=head2 config($configfile)
 
     Read configuration from configfile.
 
@@ -115,7 +137,32 @@ sub config
     return %config;
 }
 
-=head1 run_as($userid)
+
+### crc32
+sub crc32 {
+    my ($input, $init_value, $polynomial) = @_;
+    $init_value = 0 unless (defined $init_value);
+    $polynomial = 0xedb88320 unless (defined $polynomial);
+
+    my @lookup_table;
+    for (my $i=0; $i<256; $i++) {
+        my $x = $i;
+        for (my $j=0; $j<8; $j++) {
+            if ($x & 1) { $x = ($x >> 1) ^ $polynomial; }
+            else { $x = $x >> 1; }
+        }
+        push @lookup_table, $x;
+    }
+    my $crc = $init_value ^ 0xffffffff;
+    foreach my $x (unpack ('C*', $input)) {
+        $crc = (($crc >> 8) & 0xffffff) ^ $lookup_table[ ($crc ^ $x) & 0xff ];
+    }
+    $crc = $crc ^ 0xffffffff;
+    return $crc;
+}
+
+
+=head2 run_as($userid)
 
     Run command as user.
 
